/*
%	TORAST . C
%
%	convert others to SUN RASTer
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

This software is copyright (C) by the Lawrence Berkeley Laboratory.
Permission is granted to reproduce this software for non-commercial
purposes provided that this notice is left intact.

It is acknowledged that the U.S. Government has rights to this software
under Contract DE-AC03-765F00098 between the U.S.  Department of Energy
and the University of California.

This software is provided as a professional and academic contribution
for joint exchange. Thus, it is experimental, and is provided ``as is'',
with no warranties of any kind whatsoever, no support, no promise of
updates, or printed documentation. By using this software, you
acknowledge that the Lawrence Berkeley Laboratory and Regents of the
University of California shall have no liability with respect to the
infringement of other copyrights by any part of this software.

For further information about this notice, contact William Johnston,
Bld. 50B, Rm. 2239, Lawrence Berkeley Laboratory, Berkeley, CA, 94720.
(wejohnston@lbl.gov)

For further information about this software, contact:
	Jin Guojun
	Bld. 50B, Rm. 2275, Lawrence Berkeley Laboratory, Berkeley, CA, 94720.
	g_jin@lbl.gov

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% AUTHOR:	Jin Guojun - LBL	12/12/1991
*/

#include "header.def"
#include "imagedef.h"

arg_fmt_list_string	arg_fmt[] =	{
	{"-a", "%b", True, 1, 0, "add alpha channel"},
	{"-8", "%+ %d", 256, 2, 0,
		"quantizing to 8-bit with # colors [default # 256]"},
	{"-d", "%-", No, 1, 0, "dither to 8"},
	{"-k", "%b", True, 1, 0, "keep working, and ignore input error"},
	{"-l", "%-", No, 1, 0, "rotate left 90"},
	{"-r", "%+", No, 1, 0, "rotate right 90"},
	{"-u", "%N", RT_STANDARD, 1, 0,
		"use uncompress mode. The default is RLE"},
	{"-o", "%s", No, 1, 1, "output file"},
	{"	[<] input [[> | -o] output]", NULL, 0, 0, 0, "end of usage"},
	NULL	};
U_IMAGE	uimg;
bool	rot, alpha, nostop, pr_t, to8, toN=256;

#define	rows	uimg.height
#define	cols	uimg.width

main(ac, av)
int	ac;
char*	av[];
{
int	i;
char	**fl, *of_name;

	if ((i=parse_argus(&fl, ac, av, arg_fmt,
		&alpha, &to8, &toN, &to8,
		&nostop, &rot, &rot, &pr_t, &of_name)) < 0)
		exit(i);

	if (of_name && !(out_fp=freopen(av[++i], "wb", stdout)))
		syserr("output file %s", of_name);
	if (i && !(in_fp=freopen(uimg.name=fl[0], "rb", stdin)))
		syserr("input %s", fl[0]);

uimg.color_dpy = alpha ? 1 : -1;
format_init(&uimg, IMAGE_INIT_TYPE, RLE, RAS, *av, "A8-2");

io_test(fileno(in_fp), {parse_usage(arg_fmt); exit(0);});

if ((*uimg.header_handle)(HEADER_READ, &uimg, 0, 0, True))
	syserr("unknown image type");

(*uimg.std_swif)(FI_LOAD_FILE, &uimg, nostop ? NULL : uimg.name, True);

    if (rot++) {
	uimg.dest = uimg.src;
	uimg.src = nzalloc(uimg.channels*uimg.width, uimg.height, "rot");
	i = uimg.width;
	uimg.width = uimg.height;
	uimg.height = i;
	color_rotate_90(uimg.dest, uimg.src, i, uimg.width,
		uimg.color_form, rot);
	free(uimg.dest);	uimg.dest = NULL;
    }
    if (uimg.channels>1 && to8)
	To_8(&uimg, reg_cmap, to8>0, toN);

(*uimg.std_swif)(FI_SAVE_FILE, &uimg, pr_t, alpha);
exit(0);
}
